<?php

namespace Drupal\media_contextual_crop\PathProcessor;

use Drupal\image\PathProcessor\PathProcessorImageStyles as BaseFromImage;
use Symfony\Component\HttpFoundation\Request;
use Drupal\crop\CropInterface;

/**
 * Defines a path processor to rewrite contextualized image styles URLs.
 *
 * @see \Drupal\image\PathProcessor\PathProcessorImageStyles;
 */
class PathProcessorImageStyles extends BaseFromImage {

  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {

    $directory_path = $this->streamWrapperManager->getViaScheme('public')->getDirectoryPath();
    if (strpos($path, '/' . $directory_path . '/contextual/') === 0) {
      $path_prefix = '/' . $directory_path . '/contextual/';
    }
    // Check if the string '/system/files/styles/' exists inside the path,
    // that means we have a case of private file's image style.
    elseif (str_contains($path, '/system/files/contextual/')) {
      $path_prefix = '/system/files/contextual/';
      $path = substr($path, strpos($path, $path_prefix), strlen($path));
    }
    else {
      return $path;
    }

    // Strip out path prefix.
    $rest = preg_replace('|^' . preg_quote($path_prefix, '|') . '|', '', $path);

    // Get the image style, scheme and path.
    if (substr_count($rest, '/') >= 4) {
      [$style_separator, $image_style, $context, $scheme, $file] = explode('/', $rest, 5);

      // Set the file as query parameter.
      // Needed for private files
      $request->query->set('file', $file);
      return $path_prefix . $style_separator . '/' . $image_style . '/' . $context . '/' . $scheme;
    }
    // Set file query parameter if its still missing from the crop ID.
    if (substr_count($rest, '/') == 3 && !$request->query->get('file')) {
      [$style_separator, $image_style, $crop_id, $scheme] = explode('/', $rest, 4);
      $crop = \Drupal::entityTypeManager()->getStorage('crop')->load($crop_id);
      if ($crop instanceof CropInterface) {
        $request->query->set('file', $crop->uri->value);
      }
    }
    return $path;
  }

}
